@@ -31,6 +31,8 @@ DEALINGS IN THE SOFTWARE.
 #include "oxygenstyleconfig.moc"
 #include "oxygenanimationconfigwidget.h"
 #include "oxygenstyleconfigdata.h"
+#include "../oxygen.h"
+#include "config-liboxygen.h"
 
 #include <QTextStream>
 #include <QDBusMessage>
@@ -62,7 +64,6 @@ namespace Oxygen
         setupUi(this);
 
         // connections
-        connect( _windowDragMode, SIGNAL(currentIndexChanged(int)), SLOT(windowDragModeChanged(int)) );
         connect( _expertModeButton, SIGNAL(pressed()), SLOT(toggleExpertModeInternal()) );
 
         _expertModeButton->setIcon( QIcon::fromTheme( QStringLiteral( "configure" ) ) );
@@ -75,14 +76,10 @@ namespace Oxygen
 
         connect( _useBackgroundGradient, SIGNAL(toggled(bool)), SLOT(updateChanged()) );
         connect( _toolBarDrawItemSeparator, SIGNAL(toggled(bool)), SLOT(updateChanged()) );
-        connect( _checkDrawX, SIGNAL(toggled(bool)), SLOT(updateChanged()) );
         connect( _splitterProxyEnabled, SIGNAL(toggled(bool)), SLOT(updateChanged()) );
         connect( _mnemonicsMode, SIGNAL(currentIndexChanged(int)), SLOT(updateChanged()) );
         connect( _animationsEnabled, SIGNAL(toggled(bool)), SLOT(updateChanged()) );
-        connect( _cacheEnabled, SIGNAL(toggled(bool)), SLOT(updateChanged()) );
-        connect( _viewDrawTriangularExpander, SIGNAL(toggled(bool)), SLOT(updateChanged()) );
         connect( _viewTriangularExpanderSize, SIGNAL(currentIndexChanged(int)), SLOT(updateChanged()) );
-        connect( _viewDrawFocusIndicator, SIGNAL(toggled(bool)), SLOT(updateChanged()) );
         connect( _viewDrawTreeBranchLines, SIGNAL(toggled(bool)), SLOT(updateChanged()) );
         connect( _scrollBarWidth, SIGNAL(valueChanged(int)), SLOT(updateChanged()) );
         connect( _scrollBarAddLineButtons, SIGNAL(currentIndexChanged(int)), SLOT(updateChanged()) );
@@ -90,10 +87,7 @@ namespace Oxygen
         connect( _menuHighlightDark, SIGNAL(toggled(bool)), SLOT(updateChanged()) );
         connect( _menuHighlightStrong, SIGNAL(toggled(bool)), SLOT(updateChanged()) );
         connect( _menuHighlightSubtle, SIGNAL(toggled(bool)), SLOT(updateChanged()) );
-        connect( _tabStylePlain, SIGNAL(toggled(bool)), SLOT(updateChanged()) );
-        connect( _tabStyleSingle, SIGNAL(toggled(bool)), SLOT(updateChanged()) );
         connect( _windowDragMode, SIGNAL(currentIndexChanged(int)), SLOT(updateChanged()) );
-        connect( _useWMMoveResize, SIGNAL(toggled(bool)), SLOT(updateChanged()) );
 
     }
 
@@ -102,20 +96,16 @@ namespace Oxygen
     {
         StyleConfigData::setUseBackgroundGradient( _useBackgroundGradient->isChecked() );
         StyleConfigData::setToolBarDrawItemSeparator( _toolBarDrawItemSeparator->isChecked() );
-        StyleConfigData::setCheckBoxStyle( ( _checkDrawX->isChecked() ? StyleConfigData::CS_X : StyleConfigData::CS_CHECK ) );
         StyleConfigData::setSplitterProxyEnabled( _splitterProxyEnabled->isChecked() );
         StyleConfigData::setMnemonicsMode( _mnemonicsMode->currentIndex() );
-        StyleConfigData::setCacheEnabled( _cacheEnabled->isChecked() );
-        StyleConfigData::setViewDrawTriangularExpander( _viewDrawTriangularExpander->isChecked() );
         StyleConfigData::setViewTriangularExpanderSize( triangularExpanderSize() );
-        StyleConfigData::setViewDrawFocusIndicator( _viewDrawFocusIndicator->isChecked() );
         StyleConfigData::setViewDrawTreeBranchLines( _viewDrawTreeBranchLines->isChecked() );
         StyleConfigData::setScrollBarWidth( _scrollBarWidth->value() );
         StyleConfigData::setScrollBarAddLineButtons( _scrollBarAddLineButtons->currentIndex() );
         StyleConfigData::setScrollBarSubLineButtons( _scrollBarSubLineButtons->currentIndex() );
         StyleConfigData::setMenuHighlightMode( menuMode() );
-        StyleConfigData::setTabStyle( tabStyle() );
         StyleConfigData::setViewTriangularExpanderSize( triangularExpanderSize() );
+        StyleConfigData::setWindowDragMode( _windowDragMode->currentIndex()  );
 
         if( _expertMode )
         {
@@ -128,20 +118,11 @@ namespace Oxygen
 
         }
 
-        StyleConfigData::setUseWMMoveResize( _useWMMoveResize->isChecked() );
-        if( _windowDragMode->currentIndex() == 0 )
-        {
-
-            StyleConfigData::setWindowDragEnabled( false  );
-
-        } else {
-
-            StyleConfigData::setWindowDragEnabled( true  );
-            StyleConfigData::setWindowDragMode( windowDragMode()  );
-
-        }
-
+        #if OXYGEN_USE_KDE4
         StyleConfigData::self()->writeConfig();
+        #else
+        StyleConfigData::self()->save();
+        #endif
 
         // emit dbus signal
         QDBusMessage message( QDBusMessage::createSignal( QStringLiteral( "/OxygenStyle" ),  QStringLiteral( "org.kde.Oxygen.Style" ), QStringLiteral( "reparseConfiguration" ) ) );
@@ -159,7 +140,13 @@ namespace Oxygen
     //__________________________________________________________________
     void StyleConfig::reset( void )
     {
+        // reparse configuration
+        #if OXYGEN_USE_KDE4
         StyleConfigData::self()->readConfig();
+        #else
+        StyleConfigData::self()->load();
+        #endif
+
         load();
     }
 
@@ -208,8 +195,6 @@ namespace Oxygen
         _mnemonicsLabel->setVisible( _expertMode );
         _mnemonicsMode->setVisible( _expertMode );
         _animationsEnabled->setVisible( !_expertMode );
-        _cacheEnabled->setVisible( _expertMode );
-        _useWMMoveResize->setVisible( _expertMode );
         _viewsExpertWidget->setVisible( _expertMode );
 
         updateMinimumSize();
@@ -275,43 +260,16 @@ namespace Oxygen
         if ( _useBackgroundGradient->isChecked() != StyleConfigData::useBackgroundGradient() ) modified = true;
         if ( _toolBarDrawItemSeparator->isChecked() != StyleConfigData::toolBarDrawItemSeparator() ) modified = true;
         else if( _mnemonicsMode->currentIndex() != StyleConfigData::mnemonicsMode() ) modified = true;
-        else if( _viewDrawTriangularExpander->isChecked() != StyleConfigData::viewDrawTriangularExpander() ) modified = true;
-        else if( _viewDrawFocusIndicator->isChecked() != StyleConfigData::viewDrawFocusIndicator() ) modified = true;
         else if( _viewDrawTreeBranchLines->isChecked() != StyleConfigData::viewDrawTreeBranchLines() ) modified = true;
         else if( _scrollBarWidth->value() != StyleConfigData::scrollBarWidth() ) modified = true;
         else if( _scrollBarAddLineButtons->currentIndex() != StyleConfigData::scrollBarAddLineButtons() ) modified = true;
         else if( _scrollBarSubLineButtons->currentIndex() != StyleConfigData::scrollBarSubLineButtons() ) modified = true;
         else if( _splitterProxyEnabled->isChecked() != StyleConfigData::splitterProxyEnabled() ) modified = true;
-        else if( (_checkDrawX->isChecked() ? StyleConfigData::CS_X : StyleConfigData::CS_CHECK) != StyleConfigData::checkBoxStyle() ) modified = true;
         else if( menuMode() != StyleConfigData::menuHighlightMode() ) modified = true;
-        else if( tabStyle() != StyleConfigData::tabStyle() ) modified = true;
         else if( _animationsEnabled->isChecked() != StyleConfigData::animationsEnabled() ) modified = true;
-        else if( _cacheEnabled->isChecked() != StyleConfigData::cacheEnabled() ) modified = true;
-        else if( _useWMMoveResize->isChecked() != StyleConfigData::useWMMoveResize() ) modified = true;
         else if( triangularExpanderSize() != StyleConfigData::viewTriangularExpanderSize() ) modified = true;
         else if( _animationConfigWidget && _animationConfigWidget->isChanged() ) modified = true;
-
-        if( !modified )
-        {
-            switch( _windowDragMode->currentIndex() )
-            {
-                case 0:
-                {
-                    if( StyleConfigData::windowDragEnabled() ) modified = true;
-                    break;
-                }
-
-                case 1:
-                case 2:
-                default:
-                {
-                    if( !StyleConfigData::windowDragEnabled() || windowDragMode() != StyleConfigData::windowDragMode() )
-                    { modified = true; }
-                    break;
-                }
-
-            }
-        }
+        else if( _windowDragMode->currentIndex() != StyleConfigData::windowDragMode() ) modified = true;
 
         emit changed(modified);
 
@@ -325,9 +283,6 @@ namespace Oxygen
         _toolBarDrawItemSeparator->setChecked( StyleConfigData::toolBarDrawItemSeparator() );
         _mnemonicsMode->setCurrentIndex( StyleConfigData::mnemonicsMode() );
         _splitterProxyEnabled->setChecked( StyleConfigData::splitterProxyEnabled() );
-        _checkDrawX->setChecked( StyleConfigData::checkBoxStyle() == StyleConfigData::CS_X );
-        _viewDrawTriangularExpander->setChecked( StyleConfigData::viewDrawTriangularExpander() );
-        _viewDrawFocusIndicator->setChecked( StyleConfigData::viewDrawFocusIndicator() );
         _viewDrawTreeBranchLines->setChecked(StyleConfigData::viewDrawTreeBranchLines() );
 
         _scrollBarWidth->setValue(
@@ -342,16 +297,8 @@ namespace Oxygen
         _menuHighlightStrong->setChecked( StyleConfigData::menuHighlightMode() == StyleConfigData::MM_STRONG );
         _menuHighlightSubtle->setChecked( StyleConfigData::menuHighlightMode() == StyleConfigData::MM_SUBTLE );
 
-        // tab style
-        _tabStyleSingle->setChecked( StyleConfigData::tabStyle() == StyleConfigData::TS_SINGLE );
-        _tabStylePlain->setChecked( StyleConfigData::tabStyle() == StyleConfigData::TS_PLAIN );
-
         _animationsEnabled->setChecked( StyleConfigData::animationsEnabled() );
-        _cacheEnabled->setChecked( StyleConfigData::cacheEnabled() );
-
-        if( !StyleConfigData::windowDragEnabled() ) _windowDragMode->setCurrentIndex(0);
-        else if( StyleConfigData::windowDragMode() == StyleConfigData::WD_MINIMAL ) _windowDragMode->setCurrentIndex(1);
-        else _windowDragMode->setCurrentIndex(2);
+        _windowDragMode->setCurrentIndex( StyleConfigData::windowDragMode() );
 
         switch( StyleConfigData::viewTriangularExpanderSize() )
         {
@@ -360,17 +307,11 @@ namespace Oxygen
             case StyleConfigData::TE_NORMAL: _viewTriangularExpanderSize->setCurrentIndex(2); break;
         }
 
-        _useWMMoveResize->setChecked( StyleConfigData::useWMMoveResize() );
-
         // animation config widget
         if( _animationConfigWidget ) _animationConfigWidget->load();
 
     }
 
-    //__________________________________________________________________
-    void StyleConfig::windowDragModeChanged( int value )
-    { _useWMMoveResize->setEnabled( value != 0 ); }
-
     //____________________________________________________________
     int StyleConfig::menuMode( void ) const
     {
@@ -379,23 +320,6 @@ namespace Oxygen
         else return StyleConfigData::MM_STRONG;
     }
 
-    //____________________________________________________________
-    int StyleConfig::tabStyle( void ) const
-    {
-        if( _tabStylePlain->isChecked() ) return StyleConfigData::TS_PLAIN;
-        else return StyleConfigData::TS_SINGLE;
-    }
-
-    //____________________________________________________________
-    int StyleConfig::windowDragMode( void ) const
-    {
-        switch( _windowDragMode->currentIndex() )
-        {
-            case 1: return StyleConfigData::WD_MINIMAL;
-            case 2: default: return StyleConfigData::WD_FULL;
-        }
-    }
-
     //____________________________________________________________
     int StyleConfig::triangularExpanderSize( void ) const
     {
