@@ -26,6 +26,8 @@
 
 #include "oxygenshadowhelper.h"
 #include "oxygenshadowhelper.moc"
+
+#include "oxygenpropertynames.h"
 #include "oxygenshadowcache.h"
 #include "oxygenstylehelper.h"
 
@@ -36,7 +38,7 @@
 #include <QTextStream>
 #include <QEvent>
 
-#if HAVE_X11
+#if OXYGEN_HAVE_X11
 #include <QX11Info>
 #endif
 
@@ -44,8 +46,6 @@ namespace Oxygen
 {
 
     const char* const ShadowHelper::netWMShadowAtomName( "_KDE_NET_WM_SHADOW" );
-    const char* const ShadowHelper::netWMForceShadowPropertyName( "_KDE_NET_WM_FORCE_SHADOW" );
-    const char* const ShadowHelper::netWMSkipShadowPropertyName( "_KDE_NET_WM_SKIP_SHADOW" );
 
     //_____________________________________________________
     ShadowHelper::ShadowHelper( QObject* parent, StyleHelper& helper ):
@@ -54,7 +54,7 @@ namespace Oxygen
         _supported( checkSupported() ),
         _shadowCache( new ShadowCache( helper ) ),
         _size( 0 )
-        #if HAVE_X11
+        #if OXYGEN_HAVE_X11
         ,_gc( 0 ),
         _atom( 0 )
         #endif
@@ -64,11 +64,11 @@ namespace Oxygen
     ShadowHelper::~ShadowHelper( void )
     {
 
-        #if HAVE_X11
+        #if OXYGEN_HAVE_X11
         if( _helper.isX11() )
         {
-            foreach( const uint32_t& value, _pixmaps  ) xcb_free_pixmap( _helper.xcbConnection(), value );
-            foreach( const uint32_t& value, _dockPixmaps  ) xcb_free_pixmap( _helper.xcbConnection(), value );
+            foreach( const uint32_t& value, _pixmaps  ) xcb_free_pixmap( _helper.connection(), value );
+            foreach( const uint32_t& value, _dockPixmaps  ) xcb_free_pixmap( _helper.connection(), value );
         }
         #endif
 
@@ -79,12 +79,12 @@ namespace Oxygen
     //______________________________________________
     void ShadowHelper::reset( void )
     {
-        #if HAVE_X11
+        #if OXYGEN_HAVE_X11
         if( _helper.isX11() )
         {
             // round pixmaps
-            foreach( const uint32_t& value, _pixmaps  ) xcb_free_pixmap( _helper.xcbConnection(), value );
-            foreach( const uint32_t& value, _dockPixmaps  ) xcb_free_pixmap( _helper.xcbConnection(), value );
+            foreach( const uint32_t& value, _pixmaps  ) xcb_free_pixmap( _helper.connection(), value );
+            foreach( const uint32_t& value, _dockPixmaps  ) xcb_free_pixmap( _helper.connection(), value );
         }
         #endif
 
@@ -156,6 +156,7 @@ namespace Oxygen
         _size = shadowCache().shadowSize();
 
         QPixmap pixmap( shadowCache().pixmap( ShadowCache::Key() ) );
+        const QSize pixmapSize( pixmap.size()/_helper.devicePixelRatio( pixmap ) );
         if( !pixmap.isNull() )
         {
             QPainter painter( &pixmap );
@@ -166,7 +167,7 @@ namespace Oxygen
         }
 
         // recreate tileset
-        _tiles = TileSet( pixmap, pixmap.width()/2, pixmap.height()/2, 1, 1 );
+        _tiles = TileSet( pixmap, pixmapSize.width()/2, pixmapSize.height()/2, 1, 1 );
 
         if( !pixmap.isNull() )
         {
@@ -178,7 +179,7 @@ namespace Oxygen
         }
 
         // recreate tileset
-        _dockTiles = TileSet( pixmap, pixmap.width()/2, pixmap.height()/2, 1, 1 );
+        _dockTiles = TileSet( pixmap, pixmapSize.width()/2, pixmapSize.height()/2, 1, 1 );
 
         // update property for registered widgets
         for( QMap<QWidget*,WId>::const_iterator iter = _widgets.constBegin(); iter != _widgets.constEnd(); ++iter )
@@ -213,7 +214,7 @@ namespace Oxygen
     {
 
         // create atom
-        #if HAVE_X11
+        #if OXYGEN_HAVE_X11
 
         // make sure we are on X11
         if( !_helper.isX11() ) return false;
@@ -223,7 +224,7 @@ namespace Oxygen
         if( !netSupportedAtom ) return false;
 
         // store connection locally
-        xcb_connection_t* connection( _helper.xcbConnection() );
+        xcb_connection_t* connection( _helper.connection() );
 
         // get property
         const uint32_t maxLength = std::string().max_size();
@@ -280,8 +281,8 @@ namespace Oxygen
     {
 
         // flags
-        if( widget->property( netWMSkipShadowPropertyName ).toBool() ) return false;
-        if( widget->property( netWMForceShadowPropertyName ).toBool() ) return true;
+        if( widget->property( PropertyNames::netWMSkipShadow ).toBool() ) return false;
+        if( widget->property( PropertyNames::netWMForceShadow ).toBool() ) return true;
 
         // menus
         if( isMenu( widget ) ) return true;
@@ -311,7 +312,7 @@ namespace Oxygen
         */
 
         // create atom
-        #if HAVE_X11
+        #if OXYGEN_HAVE_X11
         if( !_atom && _helper.isX11() ) _atom = _helper.createAtom( QLatin1String( netWMShadowAtomName ) );
         #endif
 
@@ -325,27 +326,27 @@ namespace Oxygen
             if( _dockPixmaps.empty() && _dockTiles.isValid() )
             {
 
-                _dockPixmaps.push_back( createPixmap( _dockTiles.pixmap( 1 ) ) );
-                _dockPixmaps.push_back( createPixmap( _dockTiles.pixmap( 2 ) ) );
-                _dockPixmaps.push_back( createPixmap( _dockTiles.pixmap( 5 ) ) );
-                _dockPixmaps.push_back( createPixmap( _dockTiles.pixmap( 8 ) ) );
-                _dockPixmaps.push_back( createPixmap( _dockTiles.pixmap( 7 ) ) );
-                _dockPixmaps.push_back( createPixmap( _dockTiles.pixmap( 6 ) ) );
-                _dockPixmaps.push_back( createPixmap( _dockTiles.pixmap( 3 ) ) );
-                _dockPixmaps.push_back( createPixmap( _dockTiles.pixmap( 0 ) ) );
+                _dockPixmaps.append( createPixmap( _dockTiles.pixmap( 1 ) ) );
+                _dockPixmaps.append( createPixmap( _dockTiles.pixmap( 2 ) ) );
+                _dockPixmaps.append( createPixmap( _dockTiles.pixmap( 5 ) ) );
+                _dockPixmaps.append( createPixmap( _dockTiles.pixmap( 8 ) ) );
+                _dockPixmaps.append( createPixmap( _dockTiles.pixmap( 7 ) ) );
+                _dockPixmaps.append( createPixmap( _dockTiles.pixmap( 6 ) ) );
+                _dockPixmaps.append( createPixmap( _dockTiles.pixmap( 3 ) ) );
+                _dockPixmaps.append( createPixmap( _dockTiles.pixmap( 0 ) ) );
 
             }
 
         } else if( _pixmaps.empty() && _tiles.isValid() ) {
 
-            _pixmaps.push_back( createPixmap( _tiles.pixmap( 1 ) ) );
-            _pixmaps.push_back( createPixmap( _tiles.pixmap( 2 ) ) );
-            _pixmaps.push_back( createPixmap( _tiles.pixmap( 5 ) ) );
-            _pixmaps.push_back( createPixmap( _tiles.pixmap( 8 ) ) );
-            _pixmaps.push_back( createPixmap( _tiles.pixmap( 7 ) ) );
-            _pixmaps.push_back( createPixmap( _tiles.pixmap( 6 ) ) );
-            _pixmaps.push_back( createPixmap( _tiles.pixmap( 3 ) ) );
-            _pixmaps.push_back( createPixmap( _tiles.pixmap( 0 ) ) );
+            _pixmaps.append( createPixmap( _tiles.pixmap( 1 ) ) );
+            _pixmaps.append( createPixmap( _tiles.pixmap( 2 ) ) );
+            _pixmaps.append( createPixmap( _tiles.pixmap( 5 ) ) );
+            _pixmaps.append( createPixmap( _tiles.pixmap( 8 ) ) );
+            _pixmaps.append( createPixmap( _tiles.pixmap( 7 ) ) );
+            _pixmaps.append( createPixmap( _tiles.pixmap( 6 ) ) );
+            _pixmaps.append( createPixmap( _tiles.pixmap( 3 ) ) );
+            _pixmaps.append( createPixmap( _tiles.pixmap( 0 ) ) );
 
         }
 
@@ -368,25 +369,25 @@ namespace Oxygen
         explicitly and draw the source pixmap on it.
         */
 
-        #if HAVE_X11
+        #if OXYGEN_HAVE_X11
 
         const int width( source.width() );
         const int height( source.height() );
 
         // create X11 pixmap
-        xcb_pixmap_t pixmap = xcb_generate_id( _helper.xcbConnection() );
-        xcb_create_pixmap( _helper.xcbConnection(), 32, pixmap, QX11Info::appRootWindow(), width, height );
+        xcb_pixmap_t pixmap = xcb_generate_id( _helper.connection() );
+        xcb_create_pixmap( _helper.connection(), 32, pixmap, QX11Info::appRootWindow(), width, height );
 
         // create gc
         if( !_gc )
         {
-            _gc = xcb_generate_id( _helper.xcbConnection() );
-            xcb_create_gc( _helper.xcbConnection(), _gc, pixmap, 0, 0x0 );
+            _gc = xcb_generate_id( _helper.connection() );
+            xcb_create_gc( _helper.connection(), _gc, pixmap, 0, 0x0 );
         }
 
         // create image from QPixmap and assign to pixmap
         QImage image( source.toImage() );
-        xcb_put_image( _helper.xcbConnection(), XCB_IMAGE_FORMAT_Z_PIXMAP, pixmap, _gc, image.width(), image.height(), 0, 0, 0, 32, image.byteCount(), image.constBits());
+        xcb_put_image( _helper.connection(), XCB_IMAGE_FORMAT_Z_PIXMAP, pixmap, _gc, image.width(), image.height(), 0, 0, 0, 32, image.byteCount(), image.constBits());
 
         return pixmap;
 
@@ -407,11 +408,9 @@ namespace Oxygen
         if( !widget ) return false;
         if( !_helper.isX11() ) return false;
 
-        #if HAVE_X11
+        #if OXYGEN_HAVE_X11
         #ifndef QT_NO_XRENDER
 
-        // TODO: also check for NET_WM_SUPPORTED atom, before installing shadow
-
         /*
         From bespin code. Supposibly prevent playing with some 'pseudo-widgets'
         that have winId matching some other -random- window
@@ -428,7 +427,13 @@ namespace Oxygen
         // add pixmap handles
         QVector<uint32_t> data;
         foreach( const uint32_t& value, pixmaps )
-        { data.push_back( value ); }
+        { data.append( value ); }
+
+        // get devicePixelRatio
+        // for testing purposes only
+        // const qreal devicePixelRatio( _helper.devicePixelRatio( isDockWidget ?
+        // _dockTiles.pixmap( 0 ):_tiles.pixmap( 0 ) ) );
+        const qreal devicePixelRatio( 1.0 );
 
         // add padding
         /*
@@ -438,35 +443,30 @@ namespace Oxygen
         Some special care is needed for QBalloonTip, since the later have an arrow
         */
 
-        if( isToolTip( widget ) )
+        if( isToolTip( widget ) && widget->inherits( "QBalloonTip" ) )
         {
-            if( widget->inherits( "QBalloonTip" ) )
-            {
 
-                // balloon tip needs special margins to deal with the arrow
-                int top = 0;
-                int bottom = 0;
-                widget->getContentsMargins(NULL, &top, NULL, &bottom );
+            // balloon tip needs special margins to deal with the arrow
+            int top = 0;
+            int bottom = 0;
+            widget->getContentsMargins(nullptr, &top, nullptr, &bottom );
 
-                // also need to decrement default size further due to extra hard coded round corner
-                const int size = _size - 2;
+            // also need to decrement default size further due to extra hard coded round corner
+            const int size = (_size - 2)*devicePixelRatio;
 
-                // it seems arrow can be either to the top or the bottom. Adjust margins accordingly
-                if( top > bottom ) data << size - (top - bottom) << size << size << size;
-                else data << size << size << size - (bottom - top) << size;
-
-            } else {
-                data << _size << _size << _size << _size;
-            }
+            // it seems arrow can be either to the top or the bottom. Adjust margins accordingly
+            if( top > bottom ) data << size - (top - bottom) << size << size << size;
+            else data << size << size << size - (bottom - top) << size;
 
         } else {
 
-            data << _size << _size << _size << _size;
+            const int size = _size*devicePixelRatio;
+            data << size << size << size << size;
 
         }
 
-        xcb_change_property( _helper.xcbConnection(), XCB_PROP_MODE_REPLACE, widget->winId(), _atom, XCB_ATOM_CARDINAL, 32, data.size(), data.constData() );
-        xcb_flush( _helper.xcbConnection() );
+        xcb_change_property( _helper.connection(), XCB_PROP_MODE_REPLACE, widget->winId(), _atom, XCB_ATOM_CARDINAL, 32, data.size(), data.constData() );
+        xcb_flush( _helper.connection() );
 
         return true;
 
@@ -481,11 +481,11 @@ namespace Oxygen
     void ShadowHelper::uninstallX11Shadows( QWidget* widget ) const
     {
 
-        #if HAVE_X11
+        #if OXYGEN_HAVE_X11
         if( !_supported ) return;
         if( !_helper.isX11() ) return;
         if( !( widget && widget->testAttribute(Qt::WA_WState_Created) ) ) return;
-        xcb_delete_property( _helper.xcbConnection(), widget->winId(), _atom);
+        xcb_delete_property( _helper.connection(), widget->winId(), _atom);
         #else
         Q_UNUSED( widget )
         #endif
