@@ -27,13 +27,14 @@
 // IN THE SOFTWARE.
 //////////////////////////////////////////////////////////////////////////////
 
+#include "oxygen.h"
 #include "oxygentileset.h"
+#include "config-liboxygen.h"
 
 #include <QObject>
-#include <QPointer>
 #include <QMap>
 
-#if HAVE_X11
+#if OXYGEN_HAVE_X11
 #include <xcb/xcb.h>
 #endif
 
@@ -55,8 +56,6 @@ namespace Oxygen
         //!@name property names
         //@{
         static const char* const netWMShadowAtomName;
-        static const char* const netWMForceShadowPropertyName;
-        static const char* const netWMSkipShadowPropertyName;
         //@}
 
         //! constructor
@@ -168,7 +167,7 @@ namespace Oxygen
         //! shadow size
         int _size;
 
-        #if HAVE_X11
+        #if OXYGEN_HAVE_X11
 
         //! graphical context
         xcb_gcontext_t _gc;
