@@ -21,20 +21,29 @@
  */
 
 #include "oxygenhelper.h"
+#include "liboxygen.h"
 
 #include <KColorUtils>
 #include <KColorScheme>
 
-#include <QGuiApplication>
+#if OXYGEN_USE_KDE4
+#include <KGlobalSettings>
+#endif
+
+#include <QApplication>
 #include <QWidget>
 #include <QPainter>
 #include <QTextStream>
 #include <math.h>
 
-#if HAVE_X11
+#if OXYGEN_HAVE_X11
 #include <QX11Info>
 #endif
 
+#if OXYGEN_HAVE_X11 && QT_VERSION < 0x050000
+#include <X11/Xlib-xcb.h>
+#endif
+
 namespace Oxygen
 {
 
@@ -43,45 +52,32 @@ namespace Oxygen
     const qreal Helper::_glowBias = 0.6;
 
     //____________________________________________________________________
-    Helper::Helper( KSharedConfig::Ptr config )
-        : _config( config ),
-          _isX11( false )
-    {
-        _contrast = KColorScheme::contrastF( _config );
-
-        // background contrast is calculated so that it is 0.9
-        // when KGlobalSettings contrast value of 0.7
-        _bgcontrast = qMin( 1.0, 0.9*_contrast/0.7 );
-
-        _backgroundCache.setMaxCost( 64 );
-
-        #if HAVE_X11
-        _isX11 = QGuiApplication::platformName() == QStringLiteral("xcb");
-        if( _isX11 )
-        {
-
-            // initialize xcb connection
-            _xcbConnection = QX11Info::connection();
-            _backgroundGradientAtom = createAtom( QStringLiteral( "_KDE_OXYGEN_BACKGROUND_GRADIENT" ) );
-            _backgroundPixmapAtom = createAtom( QStringLiteral( "_KDE_OXYGEN_BACKGROUND_PIXMAP" ) );
-        } else
-        {
-            _xcbConnection = Q_NULLPTR;
-            _backgroundGradientAtom = 0;
-            _backgroundPixmapAtom = 0;
-        }
-        #endif
-
-    }
+    Helper::Helper( KSharedConfig::Ptr config ):
+        _config( config )
+    { init(); }
 
     //____________________________________________________________________
-    KSharedConfigPtr Helper::config() const
+    #if OXYGEN_USE_KDE4
+    Helper::Helper( const QByteArray& name ):
+        _componentData( name, 0, KComponentData::SkipMainComponentRegistration ),
+        _config( _componentData.config() )
+    { init(); }
+    #endif
+
+    //____________________________________________________________________
+    KSharedConfig::Ptr Helper::config() const
     { return _config; }
 
     //____________________________________________________________________
     void Helper::loadConfig()
     {
+
+        #if OXYGEN_USE_KDE4
+        _contrast = KGlobalSettings::contrastF( _config );
+        #else
         _contrast = KColorScheme::contrastF( _config );
+        #endif
+
         _bgcontrast = qMin( 1.0, 0.9*_contrast/0.7 );
 
         _viewFocusBrush = KStatefulBrush( KColorScheme::View, KColorScheme::FocusColor, _config );
@@ -240,7 +236,7 @@ namespace Oxygen
 
         if( !pixmap )
         {
-            pixmap = new QPixmap( 4, 4 );
+            pixmap = new QPixmap( highDpiPixmap( 4 ) );
             pixmap->fill( Qt::transparent );
             const qreal diameter( 1.8 );
 
@@ -248,7 +244,7 @@ namespace Oxygen
             painter.setRenderHint( QPainter::Antialiasing );
             painter.setPen( Qt::NoPen );
 
-            const QPoint center( pixmap->rect().center() );
+            const QPoint center( QRect( 0, 0, 4, 4 ).center() );
 
             // light ellipse
             painter.setBrush( calcLightColor( baseColor ) );
@@ -477,10 +473,10 @@ namespace Oxygen
             gradient.setColorAt( 0.5, color );
             gradient.setColorAt( 1.0, backgroundBottomColor( color ) );
 
-            QPainter p( pixmap );
-            p.fillRect( pixmap->rect(), gradient );
+            QPainter painter( pixmap );
+            painter.fillRect( pixmap->rect(), gradient );
 
-            p.end();
+            painter.end();
 
             _backgroundCache.insert( key, pixmap );
         }
@@ -533,11 +529,11 @@ namespace Oxygen
     }
 
     //_______________________________________________________________________
-    QRegion Helper::roundedMask( const QRect& r, int left, int right, int top, int bottom ) const
+    QRegion Helper::roundedMask( const QRect& rect, int left, int right, int top, int bottom ) const
     {
         // get rect geometry
         int x, y, w, h;
-        r.getRect( &x, &y, &w, &h );
+        rect.getRect( &x, &y, &w, &h );
 
         QRegion mask( x + 4*left, y + 0*top, w-4*( left+right ), h-0*( top+bottom ) );
         mask += QRegion( x + 0*left, y + 4*top, w-0*( left+right ), h-4*( top+bottom ) );
@@ -547,6 +543,100 @@ namespace Oxygen
         return mask;
     }
 
+    //______________________________________________________________________________
+    QBitmap Helper::roundedMask( const QSize& size, Corners corners, qreal radius ) const
+    {
+        QBitmap bitmap( highDpiPixmap( size ) );
+        if( corners == 0 )
+        {
+
+            bitmap.fill( Qt::color1 );
+
+        } else {
+
+            // initialize bitmap
+            bitmap.fill( Qt::color0 );
+
+            // setup painter
+            QPainter painter( &bitmap );
+            painter.setPen( Qt::NoPen );
+            painter.setBrush( Qt::color1 );
+
+            // get path
+            const QPainterPath path( roundedPath( bitmap.rect(), corners, radius ) );
+            painter.drawPath( path );
+
+        }
+
+        return bitmap;
+    }
+
+    //______________________________________________________________________________
+    QPainterPath Helper::roundedPath( const QRect& rect, Corners corners, qreal radius ) const
+    {
+
+        QPainterPath path;
+
+        // simple cases
+        if( corners == 0 )
+        {
+
+            path.addRect( rect );
+            return path;
+
+        }
+
+        if( corners == AllCorners ) {
+
+            path.addRoundedRect( rect, radius, radius );
+            return path;
+
+        }
+
+        const QSizeF cornerSize( 2*radius, 2*radius );
+
+        // rotate counterclockwise
+        // top left corner
+        if( corners & CornerTopLeft )
+        {
+
+            path.moveTo( rect.topLeft() + QPointF( radius, 0 ) );
+            path.arcTo( QRectF( rect.topLeft(), cornerSize ), 90, 90 );
+
+        } else path.moveTo( rect.topLeft() );
+
+        // bottom left corner
+        if( corners & CornerBottomLeft )
+        {
+
+            path.lineTo( rect.bottomLeft() - QPointF( 0, radius ) );
+            path.arcTo( QRectF( rect.bottomLeft() - QPointF( 0, 2*radius ), cornerSize ), 180, 90 );
+
+        } else path.lineTo( rect.bottomLeft() );
+
+        // bottom right corner
+        if( corners & CornerBottomRight )
+        {
+
+            path.lineTo( rect.bottomRight() - QPointF( radius, 0 ) );
+            path.arcTo( QRectF( rect.bottomRight() - QPointF( 2*radius, 2*radius ), cornerSize ), 270, 90 );
+
+        } else path.lineTo( rect.bottomRight() );
+
+        // top right corner
+        if( corners & CornerTopRight )
+        {
+
+            path.lineTo( rect.topRight() + QPointF( 0, radius ) );
+            path.arcTo( QRectF( rect.topRight() - QPointF( 2*radius, 0 ), cornerSize ), 0, 90 );
+
+        } else path.lineTo( rect.topRight() );
+
+        path.closeSubpath();
+        return path;
+
+    }
+
     //______________________________________________________________________
     void Helper::drawFloatFrame(
         QPainter* p, const QRect r,
@@ -706,7 +796,6 @@ namespace Oxygen
         lg.setColorAt( 1.0, light );
         p->setPen( QPen( lg,1 ) );
 
-
         if ( orientation == Qt::Horizontal ) p->drawLine( start+offset,end+offset );
         else
         {
@@ -725,32 +814,33 @@ namespace Oxygen
         const quint64 key( ( colorKey(glow) << 32 ) | ( quint64( 256.0 * shade ) << 24 ) | size );
         TileSet *tileSet = cache->object( key );
 
-        const qreal hScale( 1 );
-        const int hSize( size*hScale );
-        const int vSize( size );
-
         if ( !tileSet )
         {
-            QPixmap pixmap( hSize*2,vSize*2 );
+            QPixmap pixmap( highDpiPixmap( size*2 ) );
             pixmap.fill( Qt::transparent );
 
-            QPainter p( &pixmap );
-            p.setRenderHints( QPainter::Antialiasing );
-            p.setPen( Qt::NoPen );
+            QPainter painter( &pixmap );
+            painter.setRenderHints( QPainter::Antialiasing );
+            painter.setPen( Qt::NoPen );
 
-            const int fixedSize( 14 );
-            p.setWindow( 0,0,fixedSize*hScale, fixedSize );
+            const int fixedSize( 14*devicePixelRatio( pixmap ) );
+            painter.setWindow( 0, 0, fixedSize, fixedSize );
 
             // draw all components
-            if( color.isValid() ) drawShadow( p, calcShadowColor( color ), 14 );
-            if( glow.isValid() ) drawOuterGlow( p, glow, 14 );
-            if( color.isValid() ) drawSlab( p, color, shade );
+            if( color.isValid() ) drawShadow( painter, calcShadowColor( color ), 14 );
+            if( glow.isValid() ) drawOuterGlow( painter, glow, 14 );
+            if( color.isValid() ) drawSlab( painter, color, shade );
 
-            p.end();
+            painter.end();
 
-            tileSet = new TileSet( pixmap, hSize, vSize, hSize, vSize, hSize-1, vSize, 2, 1 );
+            tileSet = new TileSet( pixmap,
+                size, size,
+                size, size,
+                size-1, size,
+                2, 1 );
 
             cache->insert( key, tileSet );
+
         }
         return tileSet;
     }
@@ -763,17 +853,19 @@ namespace Oxygen
 
         if ( !tileSet )
         {
-            QPixmap pixmap( size*2, size*2 );
+            QPixmap pixmap( highDpiPixmap( size*2 ) );
             pixmap.fill( Qt::transparent );
 
-            QPainter p( &pixmap );
-            p.setRenderHints( QPainter::Antialiasing );
-            p.setPen( Qt::NoPen );
-            p.setWindow( 0,0,14,14 );
+            QPainter painter( &pixmap );
+            painter.setRenderHints( QPainter::Antialiasing );
+            painter.setPen( Qt::NoPen );
+
+            const int fixedSize( 14*devicePixelRatio( pixmap ) );
+            painter.setWindow( 0, 0, fixedSize, fixedSize );
 
             // shadow
-            p.setCompositionMode( QPainter::CompositionMode_SourceOver );
-            drawInverseShadow( p, calcShadowColor( color ), 3, 8, 0.0 );
+            painter.setCompositionMode( QPainter::CompositionMode_SourceOver );
+            drawInverseShadow( painter, calcShadowColor( color ), 3, 8, 0.0 );
 
             // contrast pixel
             {
@@ -782,14 +874,14 @@ namespace Oxygen
                 blend.setColorAt( 0.5, Qt::transparent );
                 blend.setColorAt( 1.0, light );
 
-                p.setBrush( Qt::NoBrush );
-                p.setPen( QPen( blend, 1 ) );
-                p.drawRoundedRect( QRectF( 2.5, 2.5, 9, 9 ), 4.0, 4.0 );
-                p.setPen( Qt::NoPen );
+                painter.setBrush( Qt::NoBrush );
+                painter.setPen( QPen( blend, 1 ) );
+                painter.drawRoundedRect( QRectF( 2.5, 2.5, 9, 9 ), 4.0, 4.0 );
+                painter.setPen( Qt::NoPen );
             }
 
 
-            p.end();
+            painter.end();
 
             tileSet = new TileSet( pixmap, size, size, size, size, size-1, size, 2, 1 );
 
@@ -802,13 +894,13 @@ namespace Oxygen
     }
 
     //________________________________________________________________________________________________________
-    void Helper::fillSlab( QPainter& p, const QRect& rect, int size ) const
+    void Helper::fillSlab( QPainter& painter, const QRect& rect, int size ) const
     {
         const qreal s( qreal( size ) * ( 3.6 + ( 0.5 * _slabThickness ) ) / 7.0 );
         const QRectF r( QRectF( rect ).adjusted( s, s, -s, -s ) );
         if( !r.isValid() ) return;
 
-        p.drawRoundedRect( r, s/2, s/2 );
+        painter.drawRoundedRect( r, s/2, s/2 );
     }
 
     //________________________________________________________________________________________________________
@@ -851,7 +943,7 @@ namespace Oxygen
 
     //________________________________________________________________________________________________________
     void Helper::drawInverseShadow(
-        QPainter& p, const QColor& color,
+        QPainter& painter, const QColor& color,
         int pad, int size, qreal fuzz ) const
     {
 
@@ -867,14 +959,14 @@ namespace Oxygen
             shadowGradient.setColorAt( k1, alphaColor( color, a * _shadowGain ) );
         }
         shadowGradient.setColorAt( k0, alphaColor( color, 0.0 ) );
-        p.setBrush( shadowGradient );
-        p.drawEllipse( QRectF( pad-fuzz, pad-fuzz, size+fuzz*2.0, size+fuzz*2.0 ) );
+        painter.setBrush( shadowGradient );
+        painter.drawEllipse( QRectF( pad-fuzz, pad-fuzz, size+fuzz*2.0, size+fuzz*2.0 ) );
     }
 
     //____________________________________________________________________
     const QWidget* Helper::checkAutoFillBackground( const QWidget* w ) const
     {
-        if( !w ) return NULL;
+        if( !w ) return nullptr;
         if( w->autoFillBackground() ) return w;
         if( w->isWindow() ) return 0;
 
@@ -884,14 +976,14 @@ namespace Oxygen
             if( parent == w->window() ) break;
         }
 
-        return NULL;
+        return nullptr;
     }
 
     //____________________________________________________________________
     void Helper::setHasBackgroundGradient( WId id, bool value ) const
     {
 
-        #if HAVE_X11
+        #if OXYGEN_HAVE_X11
         setHasHint( id, _backgroundGradientAtom, value );
         #else
         Q_UNUSED( id );
@@ -904,7 +996,7 @@ namespace Oxygen
     bool Helper::hasBackgroundGradient( WId id ) const
     {
 
-        #if HAVE_X11
+        #if OXYGEN_HAVE_X11
         return hasHint( id, _backgroundGradientAtom );
         #else
         Q_UNUSED( id );
@@ -916,7 +1008,7 @@ namespace Oxygen
     void Helper::setHasBackgroundPixmap( WId id, bool value ) const
     {
 
-        #if HAVE_X11
+        #if OXYGEN_HAVE_X11
         setHasHint( id, _backgroundPixmapAtom, value );
         #else
         Q_UNUSED( id );
@@ -929,7 +1021,7 @@ namespace Oxygen
     bool Helper::hasBackgroundPixmap( WId id ) const
     {
 
-        #if HAVE_X11
+        #if OXYGEN_HAVE_X11
         return hasHint( id, _backgroundPixmapAtom );
         #else
         Q_UNUSED( id );
@@ -937,22 +1029,78 @@ namespace Oxygen
         #endif
     }
 
+    //______________________________________________________________________________________
+    QPixmap Helper::highDpiPixmap( int width, int height ) const
+    {
+        #if QT_VERSION >= 0x050300
+        const qreal dpiRatio( qApp->devicePixelRatio() );
+        QPixmap pixmap( width*dpiRatio, height*dpiRatio );
+        pixmap.setDevicePixelRatio( dpiRatio );
+        return pixmap;
+        #else
+        return QPixmap( width, height );
+        #endif
+    }
 
-    #if HAVE_X11
+    //______________________________________________________________________________________
+    qreal Helper::devicePixelRatio( const QPixmap& pixmap ) const
+    {
+        #if QT_VERSION >= 0x050300
+        return pixmap.devicePixelRatio();
+        #else
+        Q_UNUSED(pixmap);
+        return 1;
+        #endif
+    }
+
+    //______________________________________________________________________________
+    bool Helper::isX11( void )
+    {
+        #if OXYGEN_HAVE_X11
+        #if QT_VERSION >= 0x050000
+        static bool isX11 = QApplication::platformName() == QStringLiteral("xcb");
+        return isX11;
+        #else
+        return true;
+        #endif
+        #endif
+
+        return false;
+
+    }
+
+    #if OXYGEN_HAVE_X11
+
+    //____________________________________________________________________
+    xcb_connection_t* Helper::connection( void )
+    {
+
+        #if QT_VERSION >= 0x050000
+        return QX11Info::connection();
+        #else
+        static xcb_connection_t* connection = nullptr;
+        if( !connection )
+        {
+            Display* display = QX11Info::display();
+            if( display ) connection = XGetXCBConnection( display );
+        }
+        return connection;
+        #endif
+    }
 
     //____________________________________________________________________
     xcb_atom_t Helper::createAtom( const QString& name ) const
     {
         if( !isX11() ) return 0;
-        xcb_intern_atom_cookie_t cookie( xcb_intern_atom( _xcbConnection, false, name.size(), qPrintable( name ) ) );
-        ScopedPointer<xcb_intern_atom_reply_t> reply( xcb_intern_atom_reply( _xcbConnection, cookie, nullptr) );
+        xcb_intern_atom_cookie_t cookie( xcb_intern_atom( connection(), false, name.size(), qPrintable( name ) ) );
+        ScopedPointer<xcb_intern_atom_reply_t> reply( xcb_intern_atom_reply( connection(), cookie, nullptr) );
         return reply ? reply->atom:0;
     }
 
     #endif
 
     //______________________________________________________________________________________
-    void Helper::drawSlab( QPainter& p, const QColor& color, qreal shade )
+    void Helper::drawSlab( QPainter& painter, const QColor& color, qreal shade )
     {
 
         const QColor light( KColorUtils::shade( calcLightColor( color ), shade ) );
@@ -960,7 +1108,7 @@ namespace Oxygen
         const QColor dark( KColorUtils::shade( calcDarkColor( color ), shade ) );
 
         // bevel, part 1
-        p.save();
+        painter.save();
         const qreal y( KColorUtils::luma( base ) );
         const qreal yl( KColorUtils::luma( light ) );
         const qreal yd( KColorUtils::luma( dark ) );
@@ -973,8 +1121,8 @@ namespace Oxygen
         }
 
         bevelGradient1.setColorAt( 0.9, base );
-        p.setBrush( bevelGradient1 );
-        p.drawRoundedRect( QRectF( 3.0,3.0,8.0,8.0 ), 3.5, 3.5 );
+        painter.setBrush( bevelGradient1 );
+        painter.drawRoundedRect( QRectF( 3.0,3.0,8.0,8.0 ), 3.5, 3.5 );
 
         // bevel, part 2
         if ( _slabThickness > 0.0 )
@@ -983,23 +1131,23 @@ namespace Oxygen
             QLinearGradient bevelGradient2( 0, 6, 0, 19 );
             bevelGradient2.setColorAt( 0.0, light );
             bevelGradient2.setColorAt( 0.9, base );
-            p.setBrush( bevelGradient2 );
-            p.drawEllipse( QRectF( 3.6,3.6,6.8,6.8 ) );
+            painter.setBrush( bevelGradient2 );
+            painter.drawEllipse( QRectF( 3.6,3.6,6.8,6.8 ) );
 
         }
 
         // inside mask
-        p.setCompositionMode( QPainter::CompositionMode_DestinationOut );
-        p.setBrush( Qt::black );
+        painter.setCompositionMode( QPainter::CompositionMode_DestinationOut );
+        painter.setBrush( Qt::black );
 
         const qreal ic( 3.6 + 0.5*_slabThickness );
         const qreal is( 14.0 - 2.0*ic );
-        p.drawEllipse( QRectF( ic, ic, is, is ) );
-        p.restore();
+        painter.drawEllipse( QRectF( ic, ic, is, is ) );
+        painter.restore();
     }
 
     //___________________________________________________________________________________________
-    void Helper::drawShadow( QPainter& p, const QColor& color, int size )
+    void Helper::drawShadow( QPainter& painter, const QColor& color, int size )
     {
         const qreal m( qreal( size-2 )*0.5 );
         const qreal offset( 0.8 );
@@ -1017,15 +1165,15 @@ namespace Oxygen
         }
 
         shadowGradient.setColorAt( 1.0, alphaColor( color, 0.0 ) );
-        p.save();
-        p.setBrush( shadowGradient );
-        p.drawEllipse( QRectF( 0, 0, size, size ) );
-        p.restore();
+        painter.save();
+        painter.setBrush( shadowGradient );
+        painter.drawEllipse( QRectF( 0, 0, size, size ) );
+        painter.restore();
 
     }
 
     //_______________________________________________________________________
-    void Helper::drawOuterGlow( QPainter& p, const QColor& color, int size )
+    void Helper::drawOuterGlow( QPainter& painter, const QColor& color, int size )
     {
 
         const QRectF r( 0, 0, size, size );
@@ -1050,19 +1198,19 @@ namespace Oxygen
         }
 
         // glow
-        p.save();
-        p.setBrush( glowGradient );
-        p.drawEllipse( r );
+        painter.save();
+        painter.setBrush( glowGradient );
+        painter.drawEllipse( r );
 
         // inside mask
-        p.setCompositionMode( QPainter::CompositionMode_DestinationOut );
-        p.setBrush( Qt::black );
-        p.drawEllipse( r.adjusted( width+0.5, width+0.5, -width-1, -width-1 ) );
-        p.restore();
+        painter.setCompositionMode( QPainter::CompositionMode_DestinationOut );
+        painter.setBrush( Qt::black );
+        painter.drawEllipse( r.adjusted( width+0.5, width+0.5, -width-1, -width-1 ) );
+        painter.restore();
 
     }
 
-    #if HAVE_X11
+    #if OXYGEN_HAVE_X11
 
     //____________________________________________________________________
     void Helper::setHasHint( xcb_window_t id, xcb_atom_t atom, bool value ) const
@@ -1073,8 +1221,8 @@ namespace Oxygen
         if( !id ) return;
 
         uint32_t uLongValue( value );
-        xcb_change_property( _xcbConnection, XCB_PROP_MODE_REPLACE, id, atom, XCB_ATOM_CARDINAL, 32, 1, &uLongValue );
-        xcb_flush( _xcbConnection );
+        xcb_change_property( connection(), XCB_PROP_MODE_REPLACE, id, atom, XCB_ATOM_CARDINAL, 32, 1, &uLongValue );
+        xcb_flush( connection() );
         return;
 
     }
@@ -1087,8 +1235,8 @@ namespace Oxygen
         // check window id
         if( !id ) return false;
 
-        xcb_get_property_cookie_t cookie( xcb_get_property( _xcbConnection, 0, id, atom, XCB_ATOM_CARDINAL, 0, 1) );
-        ScopedPointer<xcb_get_property_reply_t> reply( xcb_get_property_reply( _xcbConnection, cookie, nullptr ) );
+        xcb_get_property_cookie_t cookie( xcb_get_property( connection(), 0, id, atom, XCB_ATOM_CARDINAL, 0, 1) );
+        ScopedPointer<xcb_get_property_reply_t> reply( xcb_get_property_reply( connection(), cookie, nullptr ) );
 
         return reply && xcb_get_property_value_length( reply.data() ) && reinterpret_cast<int32_t*>(xcb_get_property_value( reply.data() ) )[0];
 
@@ -1096,4 +1244,39 @@ namespace Oxygen
 
     #endif
 
+
+    //____________________________________________________________________
+    void Helper::init( void )
+    {
+
+        #if OXYGEN_USE_KDE4
+        _contrast = KGlobalSettings::contrastF( _config );
+        #else
+        _contrast = KColorScheme::contrastF( _config );
+        #endif
+
+        // background contrast is calculated so that it is 0.9
+        // when KGlobalSettings contrast value of 0.7
+        _bgcontrast = qMin( 1.0, 0.9*_contrast/0.7 );
+
+        _backgroundCache.setMaxCost( 64 );
+
+        #if OXYGEN_HAVE_X11
+        if( isX11() )
+        {
+
+            _backgroundGradientAtom = createAtom( QStringLiteral( "_KDE_OXYGEN_BACKGROUND_GRADIENT" ) );
+            _backgroundPixmapAtom = createAtom( QStringLiteral( "_KDE_OXYGEN_BACKGROUND_PIXMAP" ) );
+
+        } else {
+
+            _backgroundGradientAtom = 0;
+            _backgroundPixmapAtom = 0;
+
+        }
+
+        #endif
+
+    }
+
 }
